// .-----------------------------------------------------------------------.
// |                                                                       |
// |                          O B J E K T Y                                |
// |                                                                       |
// `-----------------------------------------------------------------------'


#ifndef _WINDOWS_
  #define _WINDOWS_

  #include<stdio.h>
  #include<string.h>
  #include<stdlib.h>
  #include<stdarg.h>
  #include<dos.h>
  #include<dir.h>

  #ifndef _KEYBOARD_
    #include"keyboard.h"
  #endif

  #ifndef _STRING_
    #include"string.h"
  #endif

  #ifndef _TYPES_
    #include"types.h"
  #endif

  #ifndef _GRAPHIC_
    #include"graphic.h"
  #endif


#define MAX_BUTTON_NAME 20

#define BT_NOTDEF -1
#define BT_CANCEL 0
#define BT_OK     1
#define BT_YES    2
#define BT_NO     3
#define BT_USERDEF 5

// standartni barvicky komponent:
#define LABEL_INK          BLACK
#define BUTTON_BACK        GREEN
#define BUTTON_ACT_BACK    GREEN
#define BUTTON_INK         BLACK
#define BUTTON_ACT_INK     WHITE
#define BUTTON_SHADOW      BLACK
#define DIALOG_BACK        LIGHTGRAY
#define DIALOG_FRAME_INK   BLUE
#define INPUTLINE_BACK     BLUE
#define INPUTLINE_INK      WHITE
#define SCROLLBAR_BACK     CYAN
#define SCROLLBAR_INK      BLUE
#define CLUSTER_BACK       CYAN
#define CLUSTER_INK        BLACK
#define CLUSTER_ACT_BACK   CYAN
#define CLUSTER_ACT_INK    WHITE


//--------------------------- TView ------------------------------------------
class TView {
  protected:
    TPoint A,B;

  public:
    TView (){};
    TView (TPoint _A,TPoint _B){
      A=_A;
      B=_B;
    };
    virtual ~TView (){};

    virtual void draw (){};
    virtual void handle_event (int Key){};
};

class TDialog;
//--------------------------- TComponent -------------------------------------

class TComponent: protected TView {
  protected:
    boolean Enabled;
    int Tag;
    uint Options;
    void (*OnClick)();
    TDialog *Owner;
  public:
    TComponent(TPoint _A, TPoint _B, uint _Options=BT_NOTDEF,
	       boolean _Enabled=TRUE, void (*_OnClick)()=NULL):
	       TView(_A,_B){
      Enabled=_Enabled;
      Tag=0;
      Options=_Options;
      OnClick=_OnClick;
      Owner=NULL;
    }
    virtual ~TComponent(){};

    boolean is_enabled(){return (Enabled);};
    void set_enabled (boolean _Enabled){Enabled=_Enabled;};

    int get_tag (){return (Tag);};
    void set_tag(int _Tag){Tag=_Tag;};
    void set_owner (TDialog *_Owner){Owner=_Owner;};

    void set_options (uint _Options){Options=_Options;};
    uint get_options(){return (Options);};

    virtual void set_active(){};
    virtual void set_inactive(){};
    virtual void handle_event (int Key){};
    virtual void draw(){};
};

//--------------------------- TLabel -----------------------------------------
class TLabel: protected TComponent {
  protected:
    string Caption;

  public:
    TLabel (TPoint _A, TPoint _B, string _Caption):
      TComponent (_A, _B, BT_NOTDEF, FALSE){
      Caption=_Caption;
    }
    virtual ~TLabel (){};
    void set_caption (string NewCaption){Caption=NewCaption;};
    virtual void draw ();
    virtual void set_active(){};
    virtual void set_inactive(){};
};

//--------------------------- TButton -----------------------------------------
class TButton: protected TComponent {
  protected:
    char Caption[MAX_BUTTON_NAME];   //  Nazev tlacitka
    byte OldAttr;                    //  Atributy pod stinem tlacitka
  public:
    TButton (TPoint _A, TPoint _B, string _Caption, uint Type,
	     void (*_OnClick)()=NULL, int _Kind=BT_NOTDEF,
	     boolean _Enabled=TRUE):
	     TComponent (_A,_B,Type,_Enabled){

      strncpy (Caption,_Caption,MAX_BUTTON_NAME);
      OnClick=_OnClick;
      OldAttr=0;
    }
    virtual ~TButton(){};
    virtual void draw ();
    virtual void set_active();
    virtual void set_inactive();
    virtual void handle_event (int Key);
};

//--------------------------- TInputLine --------------------------------------
class TInputLine: protected TComponent {
  protected:
    string Caption;   // obsah pole
    string ValidChars;// povolene vstupni znaky
    int Length;       // maximalni delka vstupniho retezce
    int FirstLetter;  // prvni viditelne pismenko
    int CurPos;       // posice kurzoru

  public:
    TInputLine (TPoint _A, TPoint _B, int _Length,
		string _Caption=NULL,
		string _ValidChars=CH_ALL,
		uint _Options=BT_NOTDEF,
		void (*_OnClick)()=NULL,
		boolean _Enabled=TRUE);

    virtual ~TInputLine(){free (Caption);};
    virtual void draw ();
    virtual void set_active(){set_cursor (CUR_INSERT);draw();};
    virtual void set_inactive(){};
    virtual void handle_event (int Key);
    void set_caption (string _Caption){strncpy(Caption,_Caption,Length);};
    string get_caption (){return (Caption);};

};

//--------------------------- TScrollBar -----------------------------------------
class TScrollBar: protected TComponent{
  protected:
    long Max;
    long Pos;
    int Value;        // skutecna poloha soupatka
    byte OldAttr;     //  Atributy pod ukazatelem

  public:
    TScrollBar (TPoint _A, TPoint _B,
		long _Max=0, long _Pos=0,
		boolean _Enabled=FALSE):
	    TComponent (_A,_B,BT_NOTDEF,_Enabled){
      OldAttr=0;
      Max=_Max;
      Pos=_Pos;
    };
    virtual ~TScrollBar (){};
    virtual void draw (){};
    virtual void calc_value (){};
    void set_max (long _Max){Max=_Max; calc_value();};
    void set_pos (long _Pos){Pos=_Pos; calc_value();};
    void set_max_pos (long _Max, long _Pos){set_max (_Max);set_pos (_Pos);};

    virtual void redraw (long _Max, long _Pos){};
};

//--------------------------- TVScrollBar -----------------------------------------
class TVScrollBar: public TScrollBar {
  public:
    TVScrollBar (TPoint _A, TPoint _B,
		 long _Max=0, long _Pos=0,
		 boolean _Enabled=FALSE):
	  TScrollBar (_A,_B,_Max,_Pos,_Enabled){
    calc_value();};
    virtual ~TVScrollBar (){};

    virtual void calc_value();
    virtual void draw ();
    virtual void redraw (long _Max, long _Pos);
};


//--------------------------- THScrollBar -----------------------------------------
class THScrollBar: public TScrollBar {
  public:
    THScrollBar (TPoint _A, TPoint _B,
		 long _Max=0, long _Pos=0,
		 boolean _Enabled=FALSE):
	  TScrollBar (_A,_B,_Max,_Pos,_Enabled){
    calc_value();};
    virtual ~THScrollBar (){};

    virtual void calc_value();
    virtual void draw ();
    virtual void redraw (long _Max, long _Pos);
};

//--------------------------- TCluster  -----------------------------------------
class TCluster: protected TComponent {
  protected:
    string *ItemsArray;
    uint Count;
    uint Selected;
    char CheckMark;
    char LeftCh, RightCh;
  public:
    TCluster (TPoint _A, TPoint _B,
	      uint _Count, string *_Array, boolean _Enabled=TRUE):
	      TComponent (_A,_B,BT_NOTDEF,_Enabled){
      Count=_Count;
      Selected=0;
      ItemsArray=_Array;
    }
    virtual ~TCluster ();
    virtual void draw();
    virtual void check_item (uint Item){};
    virtual void uncheck_item (uint Item);
    virtual void set_active (){set_cursor (CUR_INSERT);active_item (Selected);};
    virtual void set_inactive (){};
    void active_item (uint Item);
    void inactive_item (uint Item);
    virtual void handle_event (int Key);
    virtual void prepare_items ();
    boolean checked (uint Item);
    int get_count (){return (Count);};
}; /* TCluster */


//--------------------------- TCheckBoxes  -----------------------------------------
class TCheckBoxes: public TCluster {
  public:
    TCheckBoxes (TPoint _A, TPoint _B,
	      uint _Count, string *_Array, boolean _Enabled=TRUE):
      TCluster (_A,_B,_Count, _Array, _Enabled){
      CheckMark='X';
      LeftCh='['; RightCh=']';
      prepare_items();
    }
    virtual ~TCheckBoxes(){};
    virtual void check_item(uint Item);
}; /* TCheckBoxes */

//--------------------------- TRadioButtons  -----------------------------------------
class TRadioButtons: public TCluster {
  public:
    TRadioButtons (TPoint _A, TPoint _B,
	      uint _Count, string *_Array, boolean _Enabled=TRUE):
      TCluster (_A,_B,_Count, _Array, _Enabled){
      CheckMark='';
      LeftCh='('; RightCh=')';
      prepare_items();
      string Str=ItemsArray[0];
      Str[1]=CheckMark;
    }
    virtual ~TRadioButtons(){};
    virtual void check_item(uint Item);

}; /* TRadioButtons */

//--------------------------- TDialog -----------------------------------------
class TDialog: protected TView {
  protected:
    TComponent **ArrayOfComp;      // pole komponent
    int Selected;                  // index vybrane komponenty
    int Count;                     // pocet komponent
    int Result;                    // vysledek (ktere tlacitko bylo stisknuto)
    PVideoBuffer VideoBuffer;     // Pozadi
    char FrLabel[78];             // Nazev okna
    byte FrType;                  // Typ okraje
    int FrColor;

  public:
    TDialog (TPoint _A, TPoint _B,
	     int _Count, TComponent** _Array,
	     byte _FrType=FR_SINGLE,
	     string _FrLabel=NULL,
	     int _FrColor=DIALOG_FRAME_INK);
    virtual ~TDialog ();
    void save (){save_rect(A,B,VideoBuffer);};
    void restore (){restore_rect (A,B,VideoBuffer);};
    virtual void draw();
    virtual int  open();
    virtual void close();
    virtual void handle_event (int Key);
    void set_selected (int _Selected){Selected=_Selected;};
    void set_result (int _Result);
    TComponent *get_component (uint Item){return (ArrayOfComp[Item]);};
};

// dialogy
uint message_dlg (string Message, uint BCount, ...);
void init_choosefile_dlg ();
void init_find_dlg ();
void init_replace_dlg ();
int open_choosefile_dlg (string &FileName);
int open_find_dlg (string &Str, int &CS, int &WW, int &Global);
int open_replace_dlg (string &Str1, string &Str2, int &CS, int &WW, int &Global);
void get_find_dlg_opt (string &Str, int &CS, int &WW, int &Global);
void free_choosefile_dlg ();
void free_find_dlg ();
void free_replace_dlg ();
int get_error ();
void set_error (int _Error);
void free_syntax_dlg ();
int open_syntax_dlg (int &Syntax, string &FileName);
void init_syntax_dlg ();

#endif